@@ -116,20 +116,20 @@ module FileOption = struct
 		    user2_user_find u
 		  with Not_found ->
 		    lprintf_nl "file_owner %s of %s does not exist, changing to %s"
-		      u filename admin_user.user_name;
-		    admin_user
+		      u filename (admin_user ()).user_name;
+		    admin_user ()
 		end
 	    with Not_found ->
 		lprintf_nl "file_owner of %s is empty, changing to %s"
-		  filename admin_user.user_name;
-		admin_user
+		  filename (admin_user ()).user_name;
+		admin_user ()
 	  in
 	  set_file_owner file file_user;
 
 	  let file_group =
 	    let dgroup = user2_print_user_default_group file_user in
 	    try
-	      match (get_value "file_group" stringvalue_to_option) with
+	      match (get_value "file_group" value_to_stringoption) with
 		None -> None
 	      | Some g ->
 		  begin
@@ -188,7 +188,7 @@ module FileOption = struct
         ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
         ("file_release", bool_to_value impl.impl_file_release) ::
         ("file_owner", string_to_value (file_owner file).user_name) ::
-        ("file_group", option_to_stringvalue (match file_group file with Some g -> Some g.group_name | None -> None)) ::
+        ("file_group", stringoption_to_value (match file_group file with Some g -> Some g.group_name | None -> None)) ::
           (file_to_option file)
         )
           
@@ -954,6 +954,31 @@ let search_incoming_directories () =
         [default_incoming_directories]
   | l -> l
 
+let shared_directories_including_user_commit () =
+(* This function is to be used in bTInteractive.try_share_file which is not recursive.
+   Its a replacement for !!shared_directories and provides the same list, but with
+   sub-directories added based on user_commit_dir.
+   This function works without disc access to avoid overhead. *)
+  let list = ref [] in
+  List.iter (fun s ->
+    let user_commit_dir_list = ref [] in
+    if (sharing_strategy s.shdir_strategy).sharing_incoming then
+      begin
+        user2_users_iter (fun u ->
+          if u.user_commit_dir <> "" then
+            user_commit_dir_list := !user_commit_dir_list @
+              [{
+                shdir_dirname = Filename.concat s.shdir_dirname u.user_commit_dir;
+                shdir_priority = s.shdir_priority;
+                shdir_networks = s.shdir_networks;
+                shdir_strategy = s.shdir_strategy;
+              }]
+        )
+      end;
+    list := !list @ [s] @ !user_commit_dir_list
+  ) !!shared_directories;
+  !list
+
 let incoming_dir usedir ?user ?needed_space ?network () =
 
   let directories =
@@ -1088,16 +1113,15 @@ open Zip
 
 let backup_zip archive files =
   try 
-    Unix2.tryopen_write_zip archive (fun oc ->
-      List.iter (fun file ->
-	try
+    Unix2.tryopen_umask 0o066 (fun _old_umask ->
+      Unix2.tryopen_write_zip archive (fun oc ->
+        List.iter (fun file -> try
           let module U = Unix.LargeFile in
           let s = U.stat file in
 	  Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.U.st_mtime file
 	with e ->
 	  failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) file)
-      ) files);
-    (try Unix.chmod archive 0o600 with _ -> ())
+      ) files))
   with e ->
     failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) archive)
 
@@ -1105,9 +1129,9 @@ open Tar
 
 let backup_tar archive files =
   let failed_files = ref [] in
-  Unix2.tryopen_write_tar ~compress:`Gzip archive (fun otar ->
-    List.iter (fun arg ->
-      try
+  Unix2.tryopen_umask 0o066 (fun _old_umask ->
+    Unix2.tryopen_write_tar ~compress:`Gzip archive (fun otar ->
+      List.iter (fun arg -> try
 	let header, s =
 	  Unix2.tryopen_read_bin arg (fun ic ->
 	    let stat = Unix.stat arg in
@@ -1144,8 +1168,7 @@ let backup_tar archive files =
       | e ->
 	  failed_files := arg :: !failed_files;
 	  lprintf_nl "Tar: error %s in %s" (Printexc2.to_string e) arg
-    ) files);
-    (try Unix.chmod archive 0o600 with _ -> ());
+    ) files));
     if !failed_files <> [] then
       failwith (Printf.sprintf "Tar: error backing up %s"
 	(String.concat " " (List.rev !failed_files)))
@@ -1211,63 +1234,77 @@ let _ =
   option_hook max_filenames (fun _ ->
     shorten_all_file_filenames !!max_filenames
   );
+
+  let max_opened_connections_pass = ref 0 in
   option_hook max_opened_connections (fun _ ->
+
+  incr max_opened_connections_pass;
+
+(* let users see if the option is called again *)
+  let lprintf_nl s = lprintf_nl2
+      (Printf.sprintf "%s pass %d:" log_prefix !max_opened_connections_pass) s in
+
   if !verbose then lprintf_nl
     "checking max_opened_connections = %d for validity" !!max_opened_connections;
-(* original code from ./src/config/unix/MlUnix.ml
-let max_all_sockets = getdtablesize ()
-let max_sockets = max (max_all_sockets - 100) (max_all_sockets / 2)
-let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
 
-  (* ulimit open files. minimum 150, most systems have 1024 *)
-  let max_all_sockets = Unix2.c_getdtablesize () in
+(* maximum value of open sockets/files allowed *)
+  let max_all_fds = Unix2.c_getdtablesize () in
 
-  (* old max_sockets code: max (150 - 100) (150 / 2),
-     minimum number of max_opened_connections *)
-  let min_conns = 75 in
+(* ini files, dynamic libs, etc. *)
+  let reserved_fds = max CommonOptions.min_reserved_fds (max_all_fds / 50) in
 
-  if min_conns > !!max_opened_connections then begin
-    lprintf_nl "max_opened_connections is set too low (%d), raising to %d"
-      !!max_opened_connections min_conns;
-    max_opened_connections =:= min_conns
-  end;
+(* minimum number of max_opened_connections, p2p needs some sockets *)
+  let min_conns = CommonOptions.min_connections in
+(* max_conns *should* be greater than min_conns at that point, because of
+   the sanity check at start time in CommonOptions;
+   taking the max is just a safety belt from a paranoid :) *)
+  let max_conns = max min_conns
+    (max_all_fds - reserved_fds - Unix32.max_cache_size_default) in
 
-  let reserved_fds = 40 in (* ini files, dynamic libs, etc. *)
-
-  let total_files = (* maximum number of files in use at the same time *)
-    (max (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + reserved_fds
+  let print_stats verbose =
+    if verbose then begin
+      lprintf_nl "file descriptors status: total allowed (ulimit -n) %d" max_all_fds;
+      lprintf_nl "- max_opened_connections %d (%d%% indirect)"
+        !!max_opened_connections !!max_indirect_connections;
+      lprintf_nl "- file cache size %d" (Unix32.get_max_cache_size ());
+      lprintf_nl "- reserved %d" reserved_fds;
+      let s,v =
+        let v1 =
+          max_all_fds - !!max_opened_connections - (Unix32.get_max_cache_size ()) - reserved_fds
+        in
+        if v1 >= 0 then "left", v1 else "missing", (abs v1)
+      in
+      lprintf_nl "= %d descriptors %s" v s
+    end
   in
 
-  let wanted_socks = !!max_opened_connections + total_files in
+  if !!max_opened_connections < min_conns then begin
+    lprintf_nl "max_opened_connections is set too low (%d), raising to %d"
+      !!max_opened_connections min_conns;
+    print_stats true;
+    max_opened_connections =:= min_conns
+  end
+  else if !!max_opened_connections > max_conns then begin
+    lprintf_nl "max_opened_connections is set too high (%d), lowering to %d"
+      !!max_opened_connections max_conns;
+    print_stats true;
+    max_opened_connections =:= max_conns
+  end
+  else begin
+    TcpBufferedSocket.set_max_opened_connections (fun _ -> !!max_opened_connections);
 
-  if max_all_sockets < wanted_socks then
-    if max_all_sockets < total_files + min_conns then (* check if ulimit is enough to allow total_files + min_conns *)
-      begin
-        lprintf_nl "only %d file descriptors available, raise ulimit open files to at least %d"
-          max_all_sockets wanted_socks;
-        lprintf_nl "FD info: max_opened_connections %d, number of (possible) concurrent downloads %d, = %d fd needed"
-          !!max_opened_connections total_files wanted_socks;
-        CommonGlobals.exit_properly 71
-      end
-  else
-    begin
-      let new_max_opened_connections =
-        max (max_all_sockets - total_files) (max_all_sockets / 2)
-      in
-      lprintf_nl "max_opened_connections is set too high (%d), reducing to %d"
-        !!max_opened_connections new_max_opened_connections;
-      max_opened_connections =:= new_max_opened_connections;
-    end;
+    let unused_fds = max_conns - !!max_opened_connections in
 
-  if !verbose then lprintf_nl
-    "max_opened_connections %d, total_files %d, max_concurrent_downloads %d, !!files %d"
-      !!max_opened_connections total_files !!max_concurrent_downloads (List.length !!files);
+    Unix32.set_max_cache_size
+      (Unix32.max_cache_size_default + unused_fds * 75 / 100);
 
-  TcpBufferedSocket.set_max_opened_connections
-    (fun _ -> !!max_opened_connections);
+    calc_real_max_indirect_connections ();
 
-  Unix32.max_cache_size := total_files - reserved_fds;
-  calc_real_max_indirect_connections ()
+    print_stats !verbose
+  end;
+
+  if !verbose then lprintf_nl "checking max_opened_connections finished";
+  decr max_opened_connections_pass
 )
 
 let _ =
