@@ -340,6 +340,8 @@ let download_counter = ref Int64.zero
 let nshared_files = ref 0
 let nshared_bytes = ref Int64.zero
 let shared_counter = ref Int64.zero
+let has_upload = ref 0
+let upload_credit = ref 0
 
 let string_of_field t =
   match t with
@@ -458,38 +460,8 @@ let log_chat_message i num n s =
 
 let last_message_log = ref 0
 
-
-
 let debug_clients = ref Intset.empty
 
-let default_user = {
-    ui_user = CommonUserDb.admin_user;
-    ui_user_searches = [];
-    ui_last_search = None;
-    ui_last_results = [];
-    ui_http_conn = None;
-  }
-
-let ui_users = ref [default_user]
-
-let find_ui_user user =
-  let rec iter list =
-    match list with
-      [] ->
-        let u = {
-            ui_user = (CommonUserDb.user2_user_find user);
-            ui_user_searches = [];
-            ui_last_search = None;
-            ui_last_results = [];
-            ui_http_conn = None;
-          } in
-        ui_users := u :: !ui_users;
-        u
-    | u :: tail ->
-        if u.ui_user = (CommonUserDb.user2_user_find user) then u else iter tail
-  in
-  iter !ui_users
-
 (* control_: means that it is the limited bandwidth, not the unlimited one
   used by the interfaces. tcp_: the full bandwidth (limited+unlimited) *)
 
@@ -881,6 +853,38 @@ let print_command_result o result =
   else
     Printf.bprintf o.conn_buf "%s" result
 
+let discover_ip force =
+  if !!discover_ip || force then
+    begin
+      if !verbose then lprintf_nl "started IP discovery";
+      let module H = Http_client in
+      let r = { H.basic_request with
+        H.req_url = Url.of_string "http://ip.discoveryvip.com/ip.asp";
+        H.req_proxy = !CommonOptions.http_proxy;
+        H.req_max_retry = 10;
+        H.req_user_agent = get_user_agent () }
+      in
+      H.wget r (fun file ->
+        if !verbose then lprintf_nl "downloaded IP discovery page, parsing...";
+        Unix2.tryopen_read file (fun cin ->
+          try
+            while true do
+              let line = input_line cin in
+              let search_string = "Your Ip address is  " in
+                try
+                  if Str.string_match (Str.regexp ("^" ^ search_string)) line 0 then
+                    begin
+                      set_client_ip =:=
+                        Ip.of_string (String.sub line (String.length search_string)
+                          ((String.length line) - (String.length search_string)));
+                      if !verbose then lprintf_nl "discovered IP %s" (Ip.to_string !!set_client_ip)
+                    end
+                with e -> lprintf_nl "IP discovery parse error: %s" (Printexc2.to_string e)
+            done
+          with End_of_file -> ())
+      )
+    end
+
 let _ =
   Heap.add_memstat "CommonGlobals" (fun level buf ->
       let counter = ref 0 in
