@@ -24,6 +24,7 @@ open BasicSocket
 open TcpBufferedSocket
 open Options
 open Unix
+open CommonTypes
 
 let log_prefix = "[cO]"
 
@@ -127,6 +128,9 @@ let windows_sleep seconds =
   lprintf_nl "waiting %d seconds to exit..." seconds;
   Unix.sleep seconds
 
+let min_reserved_fds = 50
+let min_connections = 50
+
 let _ =
   lprintf_nl "Starting MLDonkey %s ... " Autoconf.current_version;
   let ulof_old = Unix2.c_getdtablesize () in
@@ -139,8 +143,10 @@ let _ =
   let ulof = Unix2.c_getdtablesize () in
   if ulof_old <> ulof then
     lprintf_nl "raised ulimit for open files from %d to %d" ulof_old ulof;
-  if ulof < 150 then begin
-    lprintf_nl "ulimit for open files is set to %d, at least 150 is required, exiting..." ulof;
+  let absolute_min = Unix32.max_cache_size_default +
+    min_reserved_fds + min_connections in
+  if ulof < absolute_min then begin
+    lprintf_nl "ulimit for open files is set to %d, at least %d is required, exiting..." ulof absolute_min;
     exit 2
   end;
 
@@ -473,27 +479,27 @@ let gui_port = define_option current_section ["gui_port"]
   ~desc: "The port to connect the GUI"
   ~restart: true
   "port for Graphical Interfaces"
-    int_option 4001
+    port_option 4001
 
 let gift_port = define_option current_section ["gift_port"]
   ~desc: "The port to connect for GiFT GUIs."
   ~restart: true
   "port for GiFT Graphical Interfaces interaction. It was 1213, but the default is
   now 0 for disabled, because it does not check for a password." 
-    int_option 0
+    port_option 0
 
 let http_port = define_option current_section ["http_port"]
   ~desc: "The port to connect via HTTP"
   ~public: true
   ~restart: true
   "The port used to connect to your client with a WEB browser"
-    int_option 4080
+    port_option 4080
 
 let telnet_port = define_option current_section ["telnet_port"]
   ~desc: "The port to connect via telnet"
   ~restart: true
   "port for user interaction"
-    int_option 4000
+    port_option 4000
 
 let http_bind_addr = define_expert_option current_section ["http_bind_addr"]
   ~restart: true
@@ -556,6 +562,7 @@ let verbosity = define_expert_option current_section ["verbosity"]
   com : commands by non-admin users
   act : debug activity
   bw : debug bandwidth
+  geo : debug GeoIP
   unexp : debug unexpected messages"
     string_option ""
 
@@ -932,7 +939,7 @@ let html_mods_theme = define_option current_section ["html_mods_theme"]
     string_option ""
 
 let use_html_mods o =
-  o.CommonTypes.conn_output = CommonTypes.HTML && !!html_mods
+  o.conn_output = HTML && !!html_mods
 
 let html_checkbox_vd_file_list = define_expert_option current_section ["html_checkbox_vd_file_list"]
   "Whether to use checkboxes in the WEB interface for download list"
@@ -982,6 +989,10 @@ let force_client_ip = define_option current_section ["force_client_ip"]
   ourself. Don't set this option to true if you have dynamic IP."
     bool_option false
 
+let discover_ip = define_option current_section ["discover_ip"]
+  "Use http://ip.discoveryvip.com/ip.asp to obtain WAN IP"
+    bool_option true
+
 let user_agent = define_option current_section ["user_agent"]
   "User agent string (default = \"default\")"
     string_option "default"
@@ -1008,9 +1019,11 @@ let web_infos = define_option current_section ["web_infos"]
   EXAMPLE:
     web_infos = [
     (\"server.met\", 0, \"http://www.gruk.org/server.met.gz\");
+    (\"hublist\", 0, \"http://dchublist.com/hublist.config.bz2\");
     (\"guarding.p2p\", 96, \"http://www.bluetack.co.uk/config/level1.gz\");
     (\"ocl\", 24, \"http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl\");
     (\"contact.dat\", 168, \"http://download.overnet.org/contact.dat\");
+    (\"geoip.dat\", 168, \"http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz\");
     ]
   "
     (list_option (tuple3_option (string_option, int_option, string_option)))
@@ -1025,6 +1038,8 @@ let web_infos = define_option current_section ["web_infos"]
         "http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz");
       ("nodes.gzip", 0,
         "http://update.kceasy.com/update/fasttrack/nodes.gzip");
+      ("hublist", 0,
+        "http://dchublist.com/hublist.config.bz2");
 (*
     ("slsk_boot", 0,
       "http://www.slsknet.org/slskinfo2");
@@ -1077,10 +1092,6 @@ let mtu_packet_size = define_expert_option current_section ["mtu_packet_size"]
   "The size of the MTU of a TCP/IP packet on your connection"
     int_option 1500
 
-let packet_frame_size = define_expert_option current_section ["packet_frame_size"]
-  "The size of the frame packet on your network (on my cable link, it is 250)"
-    int_option 250
-
 let minimal_packet_size = define_expert_option current_section ["minimal_packet_size"]
   "The size of the minimal packet you want mldonkey to send when data is
   available on the connection"
@@ -1105,7 +1116,7 @@ let http_proxy_server = define_option current_section ["http_proxy_server"]
 
 let http_proxy_port = define_option current_section ["http_proxy_port"]
   "Port of HTTP proxy"
-    int_option 8080
+    port_option 8080
 
 let http_proxy_tcp = define_option current_section ["http_proxy_tcp"]
   "Direct TCP connections to HTTP proxy (the proxy should support CONNECT)"
@@ -1128,7 +1139,7 @@ let smtp_server = define_option current_section ["smtp_server"]
 
 let smtp_port = define_option current_section ["smtp_port"]
   "The port to use on the mail server (default 25)"
-    int_option 25
+    port_option 25
 
 let mail = define_option current_section ["mail"]
   "Your e-mail if you want to receive mails when downloads are completed"
@@ -1270,7 +1281,9 @@ let temp_directory = define_option current_section ["temp_directory"]
 
 let share_scan_interval = define_option current_section ["share_scan_interval"]
   ~restart: true
-  "How often (in minutes) should MLDonkey scan all shared directories for new/removed files"
+  "How often (in minutes) should MLDonkey scan all shared directories for new/removed files.
+  0 to disable scanning of shared directories. Use command reshare to manually scan shares.
+  When core starts all shared directories are scanned once, independent of this option."
     int_option 1
 
 let create_file_mode = define_option current_section ["create_file_mode"]
@@ -1371,10 +1384,6 @@ let save_results = define_option current_section ["save_results"]
   "(experimental)" 
     int_option 0
 
-let use_result_history = define_expert_option current_section ["use_file_history"]
-  "keep seen files in history to allow local search (can be expensive in memory)"
-    bool_option false
-
 let buffer_writes = define_option current_section ["buffer_writes"]
   "Buffer writes and flush after buffer_writes_delay seconds (experimental)"
     bool_option false
@@ -1559,7 +1568,7 @@ let download_sample_rate = define_expert_option current_section ["download_sampl
 
 let download_sample_size = define_expert_option current_section ["download_sample_size"]
   "How many samples go into an estimate of transfer rates"
-    int_option 10
+    int_option 100
 
 let calendar = define_expert_option current_section ["calendar"]
   "This option defines a set of date at which some commands have to be executed.
@@ -1589,7 +1598,7 @@ let max_displayed_results = define_expert_option current_section ["max_displayed
 let options_version = define_expert_option current_section ["options_version"]
   ~internal: true
   "(internal option)"
-    int_option 16
+    int_option 18
 
 let max_comments_per_file = define_expert_option current_section ["max_comments_per_file"]
   "Maximum number of comments per file"
@@ -1723,7 +1732,7 @@ let _ =
     if !!min_reask_delay < 600 then min_reask_delay =:= 600
   );
   option_hook share_scan_interval (fun _ ->
-    if !!share_scan_interval < 1 then share_scan_interval =:= 1
+    if !!share_scan_interval < 0 then share_scan_interval =:= 1
   );
   option_hook global_login (fun _ ->
       let len = String.length !!global_login in
@@ -1781,16 +1790,6 @@ let _ =
       let gc_control = Gc.get () in
       Gc.set { gc_control with Gc.space_overhead = !!space_overhead };
   );
-  option_hook web_infos (fun _ ->
-      List.iter (fun remove ->
-          if List.mem remove !!web_infos then
-            web_infos =:= List2.remove remove !!web_infos
-      )
-      [
-        ("server.met", 1, "http://savannah.nongnu.org/download/mldonkey/network/servers.met");
-        ("ocl",1, "http://savannah.nongnu.org/download/mldonkey/network/peers.ocl");
-      ]
-  );
   option_hook tcpip_packet_size (fun _ ->
       TcpBufferedSocket.ip_packet_size := !!tcpip_packet_size
   );
@@ -1800,9 +1799,6 @@ let _ =
   option_hook minimal_packet_size (fun _ ->
       TcpBufferedSocket.minimal_packet_size := !!minimal_packet_size
   );
-  option_hook packet_frame_size (fun _ ->
-      TcpBufferedSocket.packet_frame_size := !!packet_frame_size
-  );
   option_hook minor_heap_size (fun _ ->
       let gc_control = Gc.get () in
       Gc.set { gc_control with Gc.minor_heap_size =
@@ -1843,6 +1839,7 @@ let verbose_supernode = ref false
 let verbose_swarming = ref false
 let verbose_activity = ref false
 let verbose_user_commands = ref false
+let verbose_geoip = ref false
 let verbose_unexpected_messages = ref false
 
 let set_all v =
@@ -1872,6 +1869,7 @@ let set_all v =
   Http_server.verbose := v;
   verbose_activity := v;
   verbose_user_commands := v;
+  Geoip.verbose := v;
   verbose_unexpected_messages := v
 
 let _ =
@@ -1908,6 +1906,7 @@ let _ =
           | "bw" -> incr BasicSocket.verbose_bandwidth
           | "unexp" -> verbose_unexpected_messages := true
           | "com" -> verbose_user_commands := true
+          | "geo" -> Geoip.verbose := true
 
           | "all" ->
 
@@ -2022,21 +2021,54 @@ let _ =
   option_hook allow_local_network (fun _ ->
       Ip.allow_local_network := !!allow_local_network)
 
-let web_infos_exists url =
-  List.exists (fun (_, _, weburl) ->
-    weburl = url) !!web_infos
+let web_infos_table = Hashtbl.create 10
 
-let web_infos_remove outdated_web_infos =
-  web_infos =:=
-  List.fold_left (fun acc owi ->
-    let (kind2, _, url2) = owi in
-    List.filter (fun (_,_,url) -> url <> url2) acc
-  ) !!web_infos outdated_web_infos
+exception Found_web_infos of web_infos
+
+let web_infos_find url =
+  let found = ref None in
+  (try
+    Hashtbl.iter (fun key w ->
+      if w.url = url then raise (Found_web_infos w)
+    ) web_infos_table
+  with Found_web_infos w -> found := Some w);
+  !found
+
+let web_infos_remove url =
+  let delete_list = ref [] in
+  Hashtbl.iter (fun key w -> 
+    if w.url = url then delete_list := !delete_list @ [key]
+  ) web_infos_table;
+  List.iter (fun key -> Hashtbl.remove web_infos_table key) !delete_list
 
 let web_infos_add kind period url =
-  let web_info = (kind,period,url) in
-  if web_infos_exists url then web_infos_remove [web_info];
-  web_infos =:=  web_info :: !!web_infos
+  (match web_infos_find url with
+  | None -> ()
+  | Some w -> web_infos_remove w.url);
+  Hashtbl.add web_infos_table (kind, period, url)
+    {
+      kind = kind;
+      period = period;
+      url = url;
+      state = None;
+    }
+
+let _ =
+(* convert list option web_infos to a hashtable for better usage *)
+  set_after_load_hook downloads_ini (fun _ ->
+    List.iter (fun (kind, period, url) ->
+      web_infos_add kind period url
+    ) !!web_infos;
+    web_infos =:= []
+  );
+  set_before_save_hook downloads_ini (fun _ ->
+    Hashtbl.iter (fun _ w ->
+      web_infos =:= !!web_infos @ [(w.kind, w.period, w.url)]
+    ) web_infos_table
+  );
+  set_after_save_hook downloads_ini (fun _ ->
+    web_infos =:= []
+  )
 
 let rec update_options () =
   let update v =
@@ -2050,14 +2082,9 @@ let rec update_options () =
       web_infos =:= List.map (fun (kind, period, url) ->
           kind, period * Date.day_in_hours, url
       ) !!web_infos;
-      web_infos =:= !!web_infos @ [
-        ("rss", 6,
-          "http://www.ed2k-it.com/forum/news_rss.php");
-        ("rss", 6,
-          "http://www.torrents.co.uk/backend.php");
-        ("rss", 6,
-          "http://varchars.com/rss/suprnova-movies.rss");
-      ];
+      web_infos_add "rss" 6 "http://www.ed2k-it.com/forum/news_rss.php";
+      web_infos_add "rss" 6 "http://www.torrents.co.uk/backend.php";
+      web_infos_add "rss" 6 "http://varchars.com/rss/suprnova-movies.rss";
       update 1
 
   | 1 ->
@@ -2066,34 +2093,19 @@ let rec update_options () =
       update 2
 
   | 2 ->
-      web_infos_remove
-        [
-          ("rss", 6,
-            "http://www.ed2k-it.com/forum/news_rss.php");
-          ("rss", 6,
-            "http://www.torrents.co.uk/backend.php");
-          ("rss", 6,
-            "http://varchars.com/rss/suprnova-movies.rss");
-        ];
-      if !!min_reask_delay = 720 then
-          min_reask_delay =:= 600;
+      web_infos_remove "http://www.ed2k-it.com/forum/news_rss.php";
+      web_infos_remove "http://www.torrents.co.uk/backend.php";
+      web_infos_remove "http://varchars.com/rss/suprnova-movies.rss";
+      if !!min_reask_delay = 720 then min_reask_delay =:= 600;
       update 3
 
   | 3 ->
-      web_infos_remove
-        [
-          ("ocl", 24,
-            "http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl");
-        ];
+      web_infos_remove "http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl";
       web_infos_add "contact.dat" 168 "http://www.overnet.org/download/contact.dat";
       update 4
 
   | 4 ->
-      web_infos_remove
-        [
-          ("server.met", 24,
-            "http://ocbmaurice.dyndns.org/pl/slist.pl/server.met?download/server-best.met");
-        ];
+      web_infos_remove "http://ocbmaurice.dyndns.org/pl/slist.pl/server.met?download/server-best.met";
       web_infos_add "server.met" 0 "http://www.gruk.org/server.met.gz";
       update 5
 
@@ -2117,38 +2129,18 @@ let rec update_options () =
       update 9
 
   | 9 ->
-      if web_infos_exists "http://www.gruk.org/server.met.gz" then
-      begin
-        web_infos_remove
-          [
-      ("server.met", 0, "http://www.gruk.org/server.met.gz")
-          ];
+      web_infos_remove "http://www.gruk.org/server.met.gz";
       web_infos_add "server.met" 0 "http://www.jd2k.com/server.met";
-      end;
       update 10
 
   | 10 ->
-      if web_infos_exists "http://www.overnet.org/download/contact.dat" then
-      begin
-        web_infos_remove
-          [
-      ("contact.dat", 672,
-        "http://www.overnet.org/download/contact.dat");
-          ];
-  web_infos_add "contact.dat" 168 "http://download.overnet.org/contact.dat";
-      end;
+      web_infos_remove "http://www.overnet.org/download/contact.dat";
+      web_infos_add "contact.dat" 168 "http://download.overnet.org/contact.dat";
       update 11
 
   | 11 ->
-      if web_infos_exists "http://www.bluetack.co.uk/config/antip2p.txt" then
-      begin
-        web_infos_remove
-          [
-      ("guarding.p2p", 96,
-        "http://www.bluetack.co.uk/config/antip2p.txt");
-          ];
-  web_infos_add "guarding.p2p" 0 "http://www.bluetack.co.uk/config/level1.gz";
-      end;
+      web_infos_remove "http://www.bluetack.co.uk/config/antip2p.txt";
+      web_infos_add "guarding.p2p" 0 "http://www.bluetack.co.uk/config/level1.gz";
       update 12
 
   | 12 ->
@@ -2156,12 +2148,8 @@ let rec update_options () =
       update 13
 
   | 13 ->
-      if web_infos_exists "http://www.jd2k.com/server.met" then
-      begin
-        web_infos_remove
-          [("server.met", 0, "http://www.jd2k.com/server.met")];
-	web_infos_add "server.met" 0 "http://www.gruk.org/server.met.gz";
-      end;
+      web_infos_remove "http://www.jd2k.com/server.met";
+      web_infos_add "server.met" 0 "http://www.gruk.org/server.met.gz";
       update 14
 
   | 14 ->
@@ -2174,4 +2162,12 @@ let rec update_options () =
         messages_filter =:= "DI-Emule|ZamBoR|Ketamine|eMule FX|AUTOMATED MESSAGE";
       update 16
 
+  | 16 ->
+      if !!download_sample_size = 10 then download_sample_size =:= 100;
+      update 17
+
+  | 17 ->
+      web_infos_add "hublist" 0 "http://dchublist.com/hublist.config.bz2";
+      update 18
+
   | _ -> ()
