@@ -88,8 +88,9 @@ let file_string_of_uid uid =
   string_of_uid_sep uid "_"
 
 exception Illegal_urn of string
-exception Torrent_already_exists
-exception Torrent_can_not_be_used
+exception Torrent_started of string
+exception Torrent_already_exists of string
+exception Torrent_can_not_be_used of string
 
 let uid_of_string s =
   let s = String.lowercase s in
@@ -365,7 +366,17 @@ type connection_control = {
   }
 
 type output_type = TEXT | HTML | ANSI | XML | XHTML
-  
+
+type connection_type = TELNET | WEB | GUI | GIFT | CALENDAR
+
+let connection_type_to_text ct =
+  match ct with
+  | TELNET -> "Telnet"
+  | WEB -> "Web"
+  | GUI -> "GUI"
+  | GIFT -> "Gift"
+  | CALENDAR -> "Calendar"
+
 type sortvd_type = 
   BySize
 | ByName
@@ -558,6 +569,7 @@ and ui_conn = {
     mutable conn_user : ui_user;
     mutable conn_width : int;
     mutable conn_height : int;
+    mutable conn_info : (connection_type * (Ip.t * int)) option;
   }
 
   
@@ -899,15 +911,23 @@ let string_of_kind kind =
     | Indirect_location (server_ip, server_port, ip, port) -> Ip.to_string ip
   with _ -> ""
 
-let string_of_kind_geo kind =
+let string_of_kind_geo kind cc =
+  let ip_cn_cc ip =
+    match cc with
+    | None ->
+        let ccode, cname = Geoip.get_country ip in
+        Ip.to_string ip, ccode, cname
+    | Some cc ->
+        Ip.to_string ip, 
+        Geoip.country_code_array.(cc),
+        Geoip.country_name_array.(cc)
+  in
   try
     match kind with
     | Known_location (ip,port) -> 
-        let cc,cn = Geoip.get_country ip in
-        (Ip.to_string ip),cc,cn
+        ip_cn_cc ip
     | Indirect_location (server_ip, server_port, ip, port) ->
-        let cc,cn = Geoip.get_country ip in
-        (Ip.to_string ip),cc,cn
+        ip_cn_cc ip
   with _ -> "","X","Country error"
 
 type brand_stat = {
@@ -961,3 +981,25 @@ let string_of_slot_kind slot_kind short =
   | SmallFileSlot -> "SmallFileSlot"
   | NormalSlot -> if short then "" else "NormalSlot"
   | PrioSlot dir -> Printf.sprintf "Prio %s" dir
+
+type swarming_strategy =
+  LinearStrategy    (* one after the other one *)
+| AdvancedStrategy
+
+type web_infos_state =
+| DownloadStarted
+| FileLoaded
+
+type web_infos = {
+  kind : string;
+  period : int;
+  url : string;
+  mutable state : web_infos_state option;
+}
+
+let string_of_web_infos_state state =
+  match state with
+  | None -> "unknown"
+  | Some DownloadStarted -> "DL started"
+  | Some FileLoaded -> "File loaded"
+
