@@ -741,6 +741,7 @@ let get_server proto s pos =
     server_addr = addr;
     server_port = port;
     server_realport = 0;
+    server_country_code = None;
     server_score = score;
     server_tags = tags;
     server_nusers = nusers;
@@ -808,6 +809,7 @@ let get_client proto s pos =
       client_type = t;
       client_tags = tags;
       client_name = name;
+      client_country_code = None;
       client_rating = rating;
       client_chat_port = chat_port;
       client_connect_time = 0;
@@ -868,6 +870,7 @@ let get_client proto s pos =
     client_type = t;
     client_tags = tags;
     client_name = name;
+    client_country_code = None;
     client_rating = rating;
     client_chat_port = 0;
     client_connect_time = connect_time;
@@ -1056,7 +1059,7 @@ let from_gui (proto : int array) opcode s =
     | 5
     | 52 ->
         if proto < 14 then
-           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user.CommonTypes.user_name, pass)
+           let pass = fst (get_string s 2) in Password ((CommonUserDb.admin_user ()).CommonTypes.user_name, pass)
         else
         let pass,pos = get_string s 2 in
         let login,pos = get_string s pos in
