@@ -37,7 +37,6 @@ open CommonTypes
 open CommonGlobals
 open CommonOptions
 open CommonUserDb
-open CommonTypes
 open Int64ops
 
 module VB = VerificationBitmap
@@ -54,7 +53,7 @@ let verify_user_admin () =
   let warning =
     "SECURITY WARNING: user admin has an empty password, use command: useradd admin password\n"
   in
-  if has_empty_password admin_user && !!allowed_ips <>
+  if has_empty_password (admin_user ()) && !!allowed_ips <>
     [(Ip.range_of_string (strings_of_option allowed_ips).option_default)] then
     begin
       lprintf_n "%s" warning;
@@ -2000,7 +1999,7 @@ let runinfo html buf o =
   tack list
     (
       "Geoip:\t\t", 
-      if !Geoip.active then "enabled, GeoLite data created by MaxMind, available from http://maxmind.com/"
+      if Geoip.active () then "enabled, GeoLite data created by MaxMind, available from http://maxmind.com/"
       else "disabled, to enable adjust web_infos in downloads.ini for automatic download"
     );
   tack list
