@@ -75,7 +75,7 @@ module ClientOption = struct
           in
           let client_port = get_value "client_port" value_to_int in
           let client_uid = get_value "client_uid" (from_value Sha1.option) in
-          let c = new_client file client_uid (client_ip, client_port) in
+          let c = new_client file client_uid (client_ip, client_port) None in
 
           c
       | _ -> failwith "Options: Not a client"
@@ -114,7 +114,6 @@ let value_to_file file_size file_state assocs =
     with _ ->
 
         let file_name = get_value "file_name" value_to_string in
-        let file_name_utf8 = Charset.to_utf8 file_name in
         let file_comment = try get_value "file_comment" value_to_string with Not_found -> "" in
         let file_id =
           try
@@ -151,7 +150,7 @@ let value_to_file file_size file_state assocs =
         let torrent = {
             torrent_name = file_name;
             torrent_filename = "";
-            torrent_name_utf8 = file_name_utf8;
+            torrent_name_utf8 = file_name;
             torrent_comment = file_comment;
             torrent_pieces = file_chunks;
             torrent_piece_size = file_piece_size;
@@ -184,7 +183,7 @@ let value_to_file file_size file_state assocs =
         file_temp
   in
   let file = new_file file_id torrent torrent_diskname
-	       file_temp file_state CommonUserDb.admin_user in
+	       file_temp file_state (CommonUserDb.admin_user ()) in
     
   let file_uploaded = try
       value_to_int64 (List.assoc "file_uploaded" assocs)
