@@ -270,7 +270,7 @@ let new_file file_id t torrent_diskname file_temp file_state user =
           file_file = file_impl;
           file_piece_size = t.torrent_piece_size;
           file_id = file_id;
-          file_name = Charset.safe_convert t.torrent_encoding t.torrent_name;
+          file_name = t.torrent_name;
           file_comment = t.torrent_comment;
           file_created_by = t.torrent_created_by;
           file_creation_date = t.torrent_creation_date;
@@ -451,6 +451,7 @@ let decode_az_style s =
       | "BX" -> Brand_bittorrentx
       | "CT" (* ctorrent *)
       | "CD" -> Brand_ctorrent
+      | "lt" (* libtorrent *)
       | "LT" -> Brand_libtorrent
       | "MT" -> Brand_moonlighttorrent
       | "SB" -> Brand_swiftbit
@@ -463,7 +464,6 @@ let decode_az_style s =
       | "XT" -> Brand_xantorrent
       | "ZT" -> Brand_ziptorrent
       | "bk" -> Brand_bitkitten
-      | "lt" -> Brand_libtorrent
       | "MP" -> Brand_moopolice
       | "UT" -> Brand_utorrent
       | "KT" -> Brand_ktorrent
@@ -481,6 +481,10 @@ let decode_az_style s =
       | "A~" -> Brand_ares
       | "AX" -> Brand_bitpump
       | "DE" -> Brand_deluge
+      | "TT" -> Brand_tuotu
+      | "XL" -> Brand_xunlei
+      | "FT" -> Brand_foxtorrent
+      | "BF" -> Brand_bitflu
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
@@ -490,6 +494,7 @@ let decode_az_style s =
           | Brand_bitpump
           | Brand_bitcomet -> (String.sub s 4 1) ^ "." ^ (String.sub s 5 2)
           (* 3.45 *)
+          | Brand_tuotu
           | Brand_utorrent -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "." ^ (String.sub s 5 1)
           (* 34.56 *)
           | Brand_ctorrent
@@ -506,6 +511,8 @@ let decode_az_style s =
           | Brand_bitrocket -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "(" ^ (String.sub s 5 2) ^ ")"
           (* v 3456 *)
           | Brand_xtorrent -> "v" ^ (strip_leading_zeroes (String.sub s 3 4))
+          (* BitFlu is too complicated YMDD (Y+M -> HEX) eg. 7224 is 2007.02.24 *)
+          | Brand_bitflu -> ""
           (* 3.4.5.6 *)
           | _ -> (dot_string (String.sub s 3 4)) 
       in
@@ -767,10 +774,24 @@ let parse_software s =
     try iter decoder_list
     with _ -> default
 
-let new_client file peer_id kind =
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        c.client_country_code <-
+          Geoip.get_country_code_option (fst c.client_host)
+    | _ -> ()
+
+let new_client file peer_id kind cc =
   try
     let c = Hashtbl.find file.file_clients kind in
+    let old_ip = fst c.client_host in
     c.client_host <- kind;
+    if old_ip <> Ip.null && old_ip <> fst c.client_host then
+      begin
+        c.client_country_code <- None;
+        check_client_country_code c
+      end;
     c
   with _ ->
       let brand, release = parse_software (Sha1.direct_to_string peer_id) in
@@ -781,6 +802,7 @@ let new_client file peer_id kind =
           client_connection_control = new_connection_control (());
           client_file = file;
           client_host = kind;
+          client_country_code = cc;
           client_choked = true;
           client_received_peer_id = false;
           client_sent_choke = false;
@@ -824,6 +846,7 @@ let new_client file peer_id kind =
           impl_client_upload = None;
         } in
       c.client_connection_control.control_min_reask <- 120;
+      check_client_country_code c;
       new_client impl;
       Hashtbl.add file.file_clients kind c;
       file.file_clients_num <- file.file_clients_num + 1;
