@@ -180,7 +180,7 @@ let op_file_print file o =
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Filename", "sr br", "Filename");
-    ("", "sr", (Charset.safe_convert file.file_encoding file.file_name)) ];
+    ("", "sr", file.file_name) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -216,7 +216,7 @@ let op_file_print file o =
 
   html_mods_td buf [
     ("Torrent Filename", "sr br", "Torrent Fname");
-    ("", "sr", (Charset.safe_convert file.file_encoding file.file_torrent_diskname)) ];
+    ("", "sr", file.file_torrent_diskname) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
@@ -224,14 +224,14 @@ let op_file_print file o =
     ("Comment", "sr br", "Comment");
     ("", "sr", match file.file_comment with
         "" -> "-"
-      | _ -> (Charset.safe_convert file.file_encoding file.file_comment)) ];
+      | _ -> file.file_comment) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Created by", "sr br", "Created by");
     ("", "sr", match file.file_created_by with
         "" -> "-"
-      | _ -> (Charset.safe_convert file.file_encoding file.file_created_by)) ];
+      | _ -> file.file_created_by) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -243,7 +243,7 @@ let op_file_print file o =
     ("Modified by", "sr br", "Modified by");
     ("", "sr", match file.file_modified_by with
         "" -> "-"
-      | _ -> (Charset.safe_convert file.file_encoding file.file_modified_by)) ];
+      | _ -> file.file_modified_by) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -372,16 +372,13 @@ let op_file_print file o =
         Printf.bprintf buf "%s, disabled (try %d): %s\n" tracker.tracker_url i s
     | _ -> Printf.bprintf buf "%s\n" tracker.tracker_url
   ) file.file_trackers;
-  let s = Charset.safe_convert file.file_encoding file.file_torrent_diskname in
-  if s <> "" then Printf.bprintf buf "Torrent diskname: %s\n" s;
-  let s = Charset.safe_convert file.file_encoding file.file_comment in
-  if s <> "" then Printf.bprintf buf "Comment: %s\n" s;
-  let s = Charset.safe_convert file.file_encoding file.file_created_by in
-  if s <> "" then Printf.bprintf buf "Created by %s\n" s;
+  if file.file_torrent_diskname <> "" then
+    Printf.bprintf buf "Torrent diskname: %s\n" file.file_torrent_diskname;
+  if file.file_comment <> "" then Printf.bprintf buf "Comment: %s\n" file.file_comment;
+  if file.file_created_by <> "" then Printf.bprintf buf "Created by %s\n" file.file_created_by;
   let s = Date.to_string (Int64.to_float file.file_creation_date) in
   if s <> "" then Printf.bprintf buf "Creation date: %s\n" s;
-  let s = Charset.safe_convert file.file_encoding file.file_modified_by in
-  if s <> "" then Printf.bprintf buf "Modified by %s\n" s;
+  if file.file_modified_by <> "" then Printf.bprintf buf "Modified by %s\n" file.file_modified_by;
   if file.file_encoding <> "" then Printf.bprintf buf "Encoding: %s\n" file.file_encoding;
   if file.file_files <> [] then Printf.bprintf buf "Subfiles: %d\n" (List.length file.file_files);
   let cntr = ref 0 in
@@ -434,7 +431,7 @@ let op_file_print_sources file o =
         ( "0", "srh br", "Client software", "Soft" ) ;
         ( "0", "srh", "IP address", "IP address" ) ;
         ( "0", "srh br ar", "Port", "Port" ) ;
-        ] @ (if !Geoip.active then [( "0", "srh br ar", "Country Code/Name", "CC" )] else []) @ [
+        ] @ (if Geoip.active () then [( "0", "srh br ar", "Country Code/Name", "CC" )] else []) @ [
         ( "1", "srh ar", "Total UL bytes to this client for all files", "tUL" ) ;
         ( "1", "srh ar br", "Total DL bytes from this client for all files", "tDL" ) ;
         ( "1", "srh ar", "Session UL bytes to this client for all files", "sUL" ) ;
@@ -477,18 +474,21 @@ let op_file_print_sources file o =
       html_mods_table_header buf "sourcesTable" "sources al" header_list;
 
       Hashtbl.iter (fun _ c ->
+          let cinfo = client_info (as_client c) in
 	  if use_html_mods o then
           Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
 
           let btos b = if b then "T" else "F" in
-          let cc,cn = Geoip.get_country (fst c.client_host) in
+          let cc,cn = Geoip.get_country_code_name cinfo.GuiTypes.client_country_code in
           let td_list = [
             ("", "sr br ar", Printf.sprintf "%d" (client_num c));
             ("", "sr br", (Sha1.to_string c.client_uid));
             ("", "sr br", Printf.sprintf "%s %s" (brand_to_string c.client_brand) c.client_release);
             ("", "sr", (Ip.to_string (fst c.client_host)));
             ("", "sr br ar", Printf.sprintf "%d" (snd c.client_host));
-            ] @ (if !Geoip.active then [( cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
+            ] @ (if Geoip.active () then 
+              [( cn, "sr br", if use_html_mods o then CommonPictures.flag_html cc else cc)]
+                 else []) @ [
             ("", "sr ar", (size_of_int64 c.client_total_uploaded));
             ("", "sr ar br", (size_of_int64 c.client_total_downloaded));
             ("", "sr ar", (size_of_int64 c.client_session_uploaded));
@@ -631,6 +631,7 @@ let op_ft_info ft =
 
 
 let load_torrent_string s user =
+  if !verbose then lprintf_nl "load_torrent_string";
   let file_id, torrent = BTTorrent.decode_torrent s in
 
   (* Save the torrent, because we later want to put
@@ -640,21 +641,22 @@ let load_torrent_string s user =
     String2.check_prefix url "http://" in
   List.iter (fun url -> if can_handle_tracker url then torrent_is_usable := true)
     (if torrent.torrent_announce_list <> [] then torrent.torrent_announce_list else [torrent.torrent_announce]);
-  if not !torrent_is_usable then raise Torrent_can_not_be_used;
+  if not !torrent_is_usable then raise (Torrent_can_not_be_used torrent.torrent_name);
 
   let torrent_diskname =
     let fs = Unix32.filesystem downloads_directory in
     let namemax =
       match Unix32.fnamelen downloads_directory with
       | None -> 0
-      | Some v -> Int64.to_int v in
+      | Some v -> v
+    in
     Filename.concat downloads_directory
     (Filename2.filesystem_compliant torrent.torrent_name fs namemax) ^ ".torrent"
     in
   if Sys.file_exists torrent_diskname then
     begin
       if !verbose then lprintf_nl "load_torrent_string: %s already exists, ignoring" torrent_diskname;
-      raise Torrent_already_exists
+      raise (Torrent_already_exists torrent.torrent_name)
     end;
   File.from_string torrent_diskname s;
 
@@ -737,11 +739,11 @@ let try_share_file torrent_diskname =
             else
               iter tail
       in
-      iter !!shared_directories
+      iter (shared_directories_including_user_commit ())
     in
 
     let file = new_file file_id torrent torrent_diskname
-        filename FileShared CommonUserDb.admin_user in
+        filename FileShared (CommonUserDb.admin_user ()) in
     if !verbose_share then lprintf_file_nl (as_file file) "Sharing file %s" filename;
     BTClients.connect_trackers file "started"
       (parse_tracker_reply file)
@@ -758,11 +760,6 @@ let try_share_file torrent_diskname =
         with _ ->
           (lprintf_nl "Failed to rename %s to %s"
               torrent_diskname new_torrent_diskname));
-(*
-      (try Sys.remove torrent_diskname with e ->
-          if !verbose_share then
-            lprintf_nl "Error: %s while removing torrent %s" (Printexc2.to_string e) s)
-*)
   | e ->
       lprintf_nl "Cannot share torrent %s for %s"
         torrent_diskname (Printexc2.to_string e)
@@ -771,13 +768,9 @@ let try_share_file torrent_diskname =
   automatically contact the tracker. *)
 let share_files _ =
   if !verbose_share then lprintf_nl "share_files";
-  List.iter (fun dir ->
-      let filenames = Unix2.list_directory dir in
-      List.iter (fun file ->
-          let filename = Filename.concat dir file in
-          try_share_file filename
-      ) filenames
-  ) [seeded_directory];
+  List.iter (fun file ->
+    try_share_file (Filename.concat seeded_directory file)
+  ) (Unix2.list_directory seeded_directory);
   let shared_files_copy = !current_files in
  (* if the torrent is gone while the file is still shared, remove the share *)
   List.iter (fun file ->
@@ -801,10 +794,10 @@ let scan_new_torrents_directory () =
     if not (Unix2.is_directory file) then
     try
       let user = fst (Unix32.owner file) in
-      load_torrent_file file (try CommonUserDb.user2_user_find user with Not_found -> CommonUserDb.admin_user);
+      load_torrent_file file (try CommonUserDb.user2_user_find user with Not_found -> CommonUserDb.admin_user ());
       (try Sys.remove file with _ -> ())
     with 
-      Torrent_can_not_be_used ->
+      Torrent_can_not_be_used _ ->
 	Unix2.rename file (Filename.concat old_directory file_basename);
 	lprintf_nl "Torrent %s does not have valid tracker URLs, moved to torrents/old ..." file_basename
     | e ->
@@ -824,7 +817,7 @@ let load_torrent_from_web r user ft =
   H.wget r (fun filename ->
       if ft_state ft = FileDownloading then begin
           load_torrent_file filename user;
-          file_cancel (as_ft ft) CommonUserDb.admin_user
+          file_cancel (as_ft ft) (CommonUserDb.admin_user ())
         end)
 
 let valid_torrent_extension url =
@@ -888,8 +881,8 @@ let op_network_parse_url url user =
             load_torrent_file url user;
             "", true
           with
-	    Torrent_already_exists -> "A torrent with this name is already in the download queue", false
-	  | Torrent_can_not_be_used -> "This torrent does not have valid tracker URLs", false
+	    Torrent_already_exists _ -> "A torrent with this name is already in the download queue", false
+	  | Torrent_can_not_be_used _ -> "This torrent does not have valid tracker URLs", false
         with e ->
           lprintf_nl "Exception %s while 2nd loading" (Printexc2.to_string e);
 	  let s = Printf.sprintf "Can not load load torrent file: %s"
@@ -907,12 +900,14 @@ let op_network_parse_url url user =
        s, false
 
 let op_client_info c =
+  check_client_country_code c;
   let module P = GuiTypes in
   let (ip,port) = c.client_host in
   { (impl_client_info c.client_client) with
 
     P.client_network = network.network_num;
     P.client_kind = Known_location (ip,port);
+    P.client_country_code = c.client_country_code;
     P.client_state = client_state (as_client c);
     P.client_type = client_type c;
     P.client_name = (Printf.sprintf "%s:%d" (Ip.to_string ip) port);
@@ -969,7 +964,7 @@ let op_client_dprint_html c o file str =
         show_emulemods_column := true
     end;
 
-    let cc,cn = Geoip.get_country (fst c.client_host) in
+    let cc,cn = Geoip.get_country_code_name cinfo.GuiTypes.client_country_code in
 
     html_mods_td buf ([
         ("", "srb ar", Printf.sprintf "%d" (client_num c));
@@ -987,7 +982,7 @@ let op_client_dprint_html c o file str =
         ("", "sr", "D");
         ("", "sr", "N");
         ("", "sr", (Ip.to_string (fst c.client_host)));
-        ] @ (if !Geoip.active then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
+        ] @ (if Geoip.active () then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
         ("", "sr ar", (size_of_int64 c.client_total_uploaded));
         ("", "sr ar", (size_of_int64 c.client_total_downloaded));
         ("", "sr ar", (size_of_int64 c.client_session_uploaded));
@@ -1213,10 +1208,11 @@ let op_gui_message s user =
       let text = String.sub s 2 (String.length s - 2) in
       if !verbose then lprintf_nl "received torrent from gui...";
       (try
-        ignore (load_torrent_string text user)
+        let file = load_torrent_string text user in
+        raise (Torrent_started file.file_name)
       with e -> (match e with
-	  Torrent_can_not_be_used -> lprintf_nl "Loading torrent from GUI: this torrent can not be used"
-	| Torrent_already_exists -> lprintf_nl "Loading torrent from GUI: this torrent is already in download queue"
+	| Torrent_can_not_be_used s -> lprintf_nl "Loading torrent from GUI: torrent %s can not be used" s
+	| Torrent_already_exists s -> lprintf_nl "Loading torrent from GUI: torrent %s is already in download queue" s
 	| _ -> ());
 	raise e)
   | 1 -> (* 34+ *)
@@ -1250,6 +1246,16 @@ let _ =
         None -> None
       | Some sh -> Some (as_shared sh)
   );
+  file_ops.op_file_download_order <- (fun file strategy ->
+      match file.file_swarmer with
+      | None -> None
+      | Some s ->
+          (match strategy with
+          (* return current strategy *)
+          | None -> Some (CommonSwarming.get_strategy s)
+          | Some strategy -> CommonSwarming.set_strategy s strategy;
+                      Some (CommonSwarming.get_strategy s))
+  );
 
   network.op_network_gui_message <- op_gui_message;
   network.op_network_connected <- op_network_connected;
@@ -1302,6 +1308,9 @@ let _ =
   client_ops.op_client_bprint <- op_client_bprint;
   client_ops.op_client_dprint <- op_client_dprint;
   client_ops.op_client_dprint_html <- op_client_dprint_html;
+  client_ops.op_client_browse <- (fun _ _ -> ());
+  client_ops.op_client_files <- (fun _ -> []);
+  client_ops.op_client_clear_files <- (fun _ -> ());
 
   CommonNetwork.register_commands commands;
 
