@@ -299,6 +299,13 @@ let get_udp_sock () =
     None -> failwith "No UDP socket"
   | Some sock -> sock
 
+let md4_of_array md4s =
+  let s = String.create ((Array.length md4s) * 16) in
+  Array.iteri (fun i v -> 
+    String.blit (Md4.direct_to_string v) 0 s (i*16) 16
+  ) md4s;
+  Md4.string s
+
 (* compute the name used to save the file *)
 
 let update_best_name file =
@@ -369,7 +376,10 @@ let new_file file_diskname file_state md4 file_size filename writable user =
           Unix32.create_sparsefile file_diskname writable
         else
 *)
-          Unix32.create_diskfile file_diskname writable
+          try
+            Unix32.create_diskfile file_diskname writable
+          with e ->
+            failwith (Printf.sprintf "Error: %s" (Printexc2.to_string e))
       in
       let file_size =
         if file_size = Int64.zero then
@@ -472,7 +482,7 @@ let low_id ip =
     | _, _, _, 0 -> true
     | _ -> false
 
-let is_black_address ip port =
+let is_black_address ip port cc =
   !!black_list && not (low_id ip) && (
 (* lprintf "is black ="; *)
     not (Ip.reachable ip) || 
@@ -480,13 +490,19 @@ let is_black_address ip port =
      | Some br -> true 
      | None -> false) ||
     (List.mem port !!port_black_list) ||
-    (match !Ip.banned ip with
+    (match !Ip.banned (ip, cc) with
         None -> false
       | Some reason ->
           if !verbose_connect then
             lprintf_nl "%s:%d blocked: %s" (Ip.to_string ip) port reason;
           true))
 
+let check_server_country_code s =
+  if Geoip.active () then
+    match s.server_country_code with
+    | None -> s.server_country_code <- Geoip.get_country_code_option s.server_ip
+    | _ -> ()
+
 let new_server ip port =
   let key = (ip) in
   try
@@ -503,6 +519,7 @@ let new_server ip port =
         server_cid = None (* client_ip None *);
         server_port = port;
         server_realport = None;
+        server_country_code = None;
         server_sock = NoConnection;
         server_search_queries = Fifo.create ();
         server_users_queries = Fifo.create ();
@@ -560,6 +577,7 @@ let new_server ip port =
       server_add server_impl;
       Heap.set_tag s tag_server;
       Hashtbl.add servers_by_key key s;
+      check_server_country_code s;
       server_must_update s;
       s
 
@@ -582,6 +600,18 @@ let remove_server ip port =
           TcpBufferedSocket.shutdown sock Closed_by_user)
   with _ -> ()
 
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        (match c.client_kind with
+        | Direct_address (ip,port) ->
+            c.client_country_code <- Geoip.get_country_code_option ip
+        | Indirect_address (_,_,_,_,real_ip) ->
+            c.client_country_code <- Geoip.get_country_code_option real_ip
+        | _ -> ())
+    | _ -> ()
+
 let dummy_client =
   let module D = DonkeyProtoClient in
   let rec c = {
@@ -590,6 +620,7 @@ let dummy_client =
       client_kind = Direct_address (Ip.null, 0);
       client_source = DonkeySources.dummy_source;
       client_ip = Ip.null;
+      client_country_code = None;
       client_md4 = Md4.null;
       client_download = None;
       client_file_queue = [];
@@ -633,17 +664,18 @@ let dummy_client =
   in
   c
 
-let create_client key =
+let create_client key cc =
   let module D = DonkeyProtoClient in
-  let s = DonkeySources.find_source_by_uid (match key with
+  let s = DonkeySources.create_source_by_uid (match key with
       Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null) 
-      | _ -> key) in
+      | _ -> key) cc in
   let rec c = {
       client_client = client_impl;
       client_kind = key;
       client_upload = None;
       client_source = s;
       client_ip = Ip.null;
+      client_country_code = cc;
       client_md4 = Md4.null;
       client_download = None;
       client_file_queue = [];
@@ -688,26 +720,49 @@ let create_client key =
   CommonClient.new_client_with_num client_impl s.DonkeySources.source_num;
   H.add clients_by_kind c;
   clients_root := c :: !clients_root;
+  check_client_country_code c;
   c
 
 exception ClientFound of client
 let find_client_by_key key =
   try
     H.iter (fun c ->
-        if (match c.client_kind with
-      Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null)
-      | _ -> c.client_kind) = (match key with
-      Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null) 
-      | _ -> key) then raise (ClientFound c)
+      if
+     (match c.client_kind with
+      | Indirect_address (server_ip, server_port, id, port, real_ip) ->
+          Indirect_address (server_ip, server_port, id, 0, Ip.null)
+      | _ -> c.client_kind) =
+     (match key with
+      | Indirect_address (server_ip, server_port, id, port, real_ip) ->
+          Indirect_address (server_ip, server_port, id, 0, Ip.null) 
+      | _ -> key) then
+        raise (ClientFound c)
     ) clients_by_kind;
     raise Not_found
   with ClientFound c -> c
 
-let new_client key =
+let new_client key cc =
   try
-      find_client_by_key key
+    let c = find_client_by_key key in
+(* An indirect client without real_ip might have been created earlier.
+   If that client connected us later we have its real ip *)
+    (match key with
+    | Indirect_address (_,_,_,_,ip_real) ->
+        let old_ip =
+          match c.client_kind with
+          | Indirect_address (_,_,_,_,old_ip) ->
+              if old_ip = Ip.null then None else Some old_ip
+          | _ -> None
+        in
+        if ip_real <> Ip.null then c.client_kind <- key;
+        (match old_ip with
+        | Some old_ip ->
+            if old_ip <> ip_real then check_client_country_code c
+        | None -> if ip_real <> Ip.null then check_client_country_code c)
+    | _ -> ());
+    c
   with _ ->
-      create_client key
+      create_client key cc
 
 let create_client = ()
 
@@ -1011,11 +1066,14 @@ let _ =
 let full_client_identifier c =
     Printf.sprintf "%s (%s%s) '%s'"
       (match c.client_kind with
-          Indirect_address (server_ip, server_port, ip, port, real_ip) ->
-            Printf.sprintf "%s:%d(lowID, server:%s:%d]"
-              (Ip.to_string real_ip) port (Ip.to_string server_ip) server_port;
+          Indirect_address (server_ip, server_port, id, port, real_ip) ->
+            Printf.sprintf "%s:%d%s[lowID %Ld, server:%s:%d]"
+              (Ip.to_string real_ip) port
+              (match c.client_country_code with | None -> "" | Some cc -> Printf.sprintf "(%d)" cc)
+              id (Ip.to_string server_ip) server_port
         | Direct_address (ip,port) ->
-            Printf.sprintf "%s:%d" (Ip.to_string ip) port;
+            Printf.sprintf "%s:%d%s" (Ip.to_string ip) port
+              (match c.client_country_code with | None -> "" | Some cc -> Printf.sprintf "(%d)" cc)
         | Invalid_address _ -> " invalid IP")
       (GuiTypes.client_software_short (brand_to_string_short c.client_brand) c.client_osinfo)
       (if c.client_emule_proto.emule_release = "" then "" else " " ^ c.client_emule_proto.emule_release)
