@@ -438,7 +438,7 @@ let import_temp temp_dir =
 		(match !filename_met with
 		   None -> filename
 		| Some s -> s) !size f.P.md4 None
-              (Some filename) (Some (List.rev f.P.absents)) CommonUserDb.admin_user);
+              (Some filename) (Some (List.rev f.P.absents)) (CommonUserDb.admin_user ()));
       with _ -> ()
   ) list
 
@@ -642,7 +642,7 @@ let parse_donkey_url url user =
   | "friend" :: ip :: port :: _ ->
       let ip = Ip.of_string ip in
       let port = int_of_string port in
-      let c = new_client (Direct_address (ip,port)) in
+      let c = new_client (Direct_address (ip,port)) None in
       friend_add c;
       "", true
 
@@ -690,18 +690,11 @@ let commands = [
         in
         let ip = Ip.from_name ip in
         let port = int_of_string port in
-        let c = new_client (Direct_address (ip,port)) in
+        let c = new_client (Direct_address (ip,port)) None in
         friend_add c;
         "friend added";
     ),  "<ip> [<port>] :\t\t\tadd a friend";
 
-    "vu", Arg_none (fun o ->
-        Printf.sprintf
-          "Upload credits : %d minutes\nUpload disabled for %d minutes"
-          !CommonUploads.upload_credit !CommonUploads.has_upload;
-
-    ), ":\t\t\t\t\tview upload credits";
-
 (*
     "comments", Arg_one (fun filename o ->
 (* TODO        DonkeyIndexer.load_comments filename;
@@ -1019,15 +1012,29 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
 
 let _ =
   register_commands commands;
+  file_ops.op_file_download_order <- (fun file strategy ->
+      match file.file_swarmer with
+      | None -> None
+      | Some s ->
+          (match strategy with
+          (* return current strategy *)
+          | None -> Some (CommonSwarming.get_strategy s)
+          | Some strategy -> CommonSwarming.set_strategy s strategy;
+                      Some (CommonSwarming.get_strategy s))
+  );
   file_ops.op_file_resume <- (fun file ->
       reconnect_all file;
   );
-  file_ops.op_file_set_priority <- (fun file _ ->
-(*  TODO PRIORITY: take care priorities in CommonSources
-DonkeySources.recompute_ready_sources ()        *)
-      ()
+  file_ops.op_file_pause <- (fun file ->
+    DonkeySources.iter_active_sources (fun s ->
+      let s_uid = s.DonkeySources.source_uid in
+      let c = new_client s_uid s.DonkeySources.source_country_code in
+      match client_state c with
+      | Connected_downloading f when f = file_num file -> disconnect_client c Closed_by_peer
+      | _ -> ()
+    ) file.file_sources
   );
-  file_ops.op_file_pause <- (fun file -> ()  );
+  file_ops.op_file_queue <- file_ops.op_file_pause;
 
   file_ops.op_file_commit <- (fun file new_name ->
 
@@ -1119,6 +1126,7 @@ let _ =
 
 let _ =
   server_ops.op_server_info <- (fun s ->
+      check_server_country_code s;
       if !!enable_donkey then
         { (impl_server_info s.server_server) with
 
@@ -1126,6 +1134,7 @@ let _ =
           P.server_addr = Ip.addr_of_ip s.server_ip;
           P.server_port = s.server_port;
           P.server_realport = (match s.server_realport with Some x -> x | None -> 0);
+    	  P.server_country_code = s.server_country_code;
           P.server_score = s.server_score;
           P.server_nusers = (match s.server_nusers with None -> 0L | Some v -> v);
           P.server_nfiles = (match s.server_nfiles with None -> 0L | Some v -> v);
@@ -1183,11 +1192,15 @@ let string_of_client_addr c =
   with _ -> ""
 
 let get_ips_cc_cn c =
+  check_client_country_code c;
   try
     match c.client_kind with
-      Direct_address (ip,port) -> 
-        let cc,cn = Geoip.get_country ip in
+    | Direct_address (ip,port) ->
+        let cc,cn = Geoip.get_country_code_name c.client_country_code in
         (Ip.to_string ip),cc,cn
+    | Indirect_address (_,_,_,_,real_ip) ->
+        let cc,cn = Geoip.get_country_code_name c.client_country_code in
+        (Ip.to_string real_ip),cc,cn
     | _ ->  
         let cc,cn = Geoip.unknown_country in
         (string_of_client_addr c),cc,cn
@@ -1196,6 +1209,7 @@ let get_ips_cc_cn c =
 
 let _ =
   client_ops.op_client_info <- (fun c ->
+      check_client_country_code c;
       { (impl_client_info c.client_client) with
 
         P.client_network = network.network_num;
@@ -1204,6 +1218,7 @@ let _ =
 	  | Indirect_address (server_ip, server_port, ip, port, real_ip) -> 
 	      Indirect_location (c.client_name,c.client_md4, real_ip, port)
           | _ -> Indirect_location (c.client_name,c.client_md4, c.client_ip, 0));
+        P.client_country_code = c.client_country_code;
         P.client_state = client_state c;
         P.client_type = client_type c;
         P.client_name = c.client_name;
@@ -1290,7 +1305,7 @@ let _ =
       let list = ref [] in
       DonkeySources.iter_all_sources (fun s ->
           let s_uid = s.DonkeySources.source_uid in
-          let c = new_client s_uid in
+          let c = new_client s_uid s.DonkeySources.source_country_code in
           list := (as_client c) :: !list
       ) file.file_sources;
       !list
@@ -1299,7 +1314,7 @@ let _ =
       let list = ref [] in
       DonkeySources.iter_active_sources (fun s ->
           let s_uid = s.DonkeySources.source_uid in
-          let c = new_client s_uid in
+          let c = new_client s_uid s.DonkeySources.source_country_code in
           list := (as_client c) :: !list
       ) file.file_sources;
       !list
@@ -1388,7 +1403,7 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
     let sources_list = ref [] in
     DonkeySources.iter_relevant_sources (fun s ->
       let s_uid = s.DonkeySources.source_uid in
-      let c = new_client s_uid in
+      let c = new_client s_uid s.DonkeySources.source_country_code in
       sources_list := (s,c) :: !sources_list
     ) file.file_sources;
 
@@ -1415,7 +1430,7 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
           ( "0", "srh", "Connection [I]ndirect, [D]irect", "C" ) ;
           ( "0", "srh", "Secure User Identification [N]one, [P]assed, [F]ailed", "S" ) ;
           ( "0", "srh br", "IP address", "IP address" ) ;
-          ] @ (if !Geoip.active then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
+          ] @ (if Geoip.active () then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
           ( "1", "srh ar", "Total UL bytes to this client for all files", "tUL" ) ;
           ( "1", "srh ar br", "Total DL bytes from this client for all files", "tDL" ) ;
           ( "1", "srh ar", "Session UL bytes to this client for all files", "sUL" ) ;
@@ -1497,7 +1512,7 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
                   | Some b -> if b then "P" else "F"
                 ));
             ("", "sr br", ip_string);
-            ] @ (if !Geoip.active then [(cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
+            ] @ (if Geoip.active () then [(cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
             ("", "sr ar", (size_of_int64 c.client_total_uploaded));
             ("", "sr ar br", (size_of_int64 c.client_total_downloaded));
             ("", "sr ar", (size_of_int64 c.client_session_uploaded));
@@ -1576,7 +1591,7 @@ let try_recover_temp_file filename md4 =
       let size = Unix32.getsize file_diskname in
       if size <> zero then
 	begin
-	  ignore (really_query_download (Md4.to_string md4) size md4 None (Some file_diskname) None CommonUserDb.admin_user);
+	  ignore (really_query_download (Md4.to_string md4) size md4 None (Some file_diskname) None (CommonUserDb.admin_user ()));
 	  recover_md4s md4
 	end
 
@@ -1717,7 +1732,7 @@ let _ =
       );
       Printf.bprintf buf "  state: %s, rank: %d\n"
         (string_of_connection_state (client_state c)) c.client_rank;
-      if !Geoip.active then Printf.bprintf buf "  country: %s: %s\n" cc cn;
+      if Geoip.active () then Printf.bprintf buf "  country: %s: %s\n" cc cn;
       Printf.bprintf buf "  MD4: %s\n" (Md4.to_string c.client_md4);
       Printf.bprintf buf "  downloaded\n";
       Printf.bprintf buf "   - session %s\n" (size_of_int64 c.client_session_downloaded);
@@ -1839,7 +1854,7 @@ let _ =
                         | Some b -> if b then "P" else "F"
                       )); 
                       ("", "sr", ip_string);
-                      ] @ (if !Geoip.active then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
+                      ] @ (if Geoip.active () then [(cn, "sr", CommonPictures.flag_html cc)] else []) @ [
                       ("", "sr ar", (size_of_int64 c.client_total_uploaded));
                       ("", "sr ar", (size_of_int64 c.client_total_downloaded));
                       ("", "sr ar", (size_of_int64 c.client_session_uploaded));
@@ -1915,23 +1930,18 @@ let _ =
     if !!enable_donkey && !!update_server_list_server_met then
       begin
         lprintf_nl "server.met loaded from %s" url;
-  begin
-    try
-      let s = unpack_server_met filename url in
+        let s = unpack_server_met filename url in
         let nservers = List.length (Hashtbl2.to_list servers_by_key) in
-              let n = load_server_met s in
-                if s <> filename then Sys.remove s;
-            lprintf_nl "%d servers found, %d new ones inserted"
-        n ((List.length (Hashtbl2.to_list servers_by_key)) - nservers)
-           with _ -> ()
-  end;
+        let n = load_server_met s in
+        if s <> filename then Sys.remove s;
+        lprintf_nl "%d servers found, %d new ones inserted"
+          n ((List.length (Hashtbl2.to_list servers_by_key)) - nservers)
       end
     else
       if not !!enable_donkey then
         lprintf_nl "eDonkey module is disabled, ignoring..."
       else
-        lprintf_nl "ED2K-update_server_list_server_met is disabled, ignoring...";
-    CommonWeb.remove_job url
+        lprintf_nl "ED2K-update_server_list_server_met is disabled, ignoring..."
   );
 
   file_ops.op_file_proposed_filenames <- op_file_proposed_filenames;
