@@ -184,6 +184,14 @@ let find_server ip port =
     h.host_server
   with _ -> None
 
+let check_server_country_code s =
+  if Geoip.active () then
+    match s.server_country_code with
+    | None ->
+        s.server_country_code <-
+          Geoip.get_country_code_option (Ip.ip_of_addr s.server_host.host_addr)
+    | _ -> ()
+
 let new_server ip port =
   let h = H.new_host ip port Ultrapeer in
   match h.host_server with
@@ -193,6 +201,7 @@ let new_server ip port =
           server_server = server_impl;
           server_ciphers = None;
           server_host = h;
+          server_country_code = None;
           server_sock = NoConnection;
           server_agent = "<unknown>";
           server_description = "";
@@ -220,6 +229,7 @@ let new_server ip port =
       server_add server_impl;
       h.host_server <- Some s;
       h.host_on_remove <- (fun _ -> server_remove (as_server server_impl));
+      check_server_country_code s;
       s
       
 let extract_uids arg = Uid.expand [Uid.of_string arg]
@@ -413,7 +423,17 @@ let new_user kind =
       user_add user_impl;
       Hashtbl.add users_by_uid kind user;
       user
-  
+
+let check_client_country_code c =
+  if Geoip.active () then
+    match c.client_country_code with
+    | None ->
+        (match c.client_host with
+        | Some (ip,port) ->
+            c.client_country_code <- Geoip.get_country_code_option ip
+        | _ -> ())
+    | _ -> ()
+
 let new_client kind =
   try
     Hashtbl.find clients_by_uid kind 
@@ -435,6 +455,7 @@ client_error = false;
           client_connection_control = new_connection_control (());
           client_downloads = [];
           client_host = None;
+          client_country_code = None;
           client_reconnect = false;
           client_in_queues = [];
           client_connected_for = None;
