@@ -17,13 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "../../utils/lib/os_stubs.h"
 #include "tiger.h"
 
-/* Big endian:                                         */
-#if !(defined(__alpha)||defined(__i386__)||defined(__vax__))
-#define BIG_ENDIAN
-#endif
-
 /* The following macro denotes that an optimization    */
 /* for Alpha is required. It is used only for          */
 /* optimization of time. Otherwise it does nothing.    */
@@ -193,7 +189,7 @@ static void static_tiger(word64 *str, word64 length, word64 res[3])
 
   for(i=length; i>=64; i-=64)
     {
-#ifdef BIG_ENDIAN
+#ifdef ARCH_BIG_ENDIAN
       for(j=0; j<64; j++)
 	temp[j^7] = ((byte*)str)[j];
       tiger_compress(((word64*)temp), res);
@@ -203,7 +199,7 @@ static void static_tiger(word64 *str, word64 length, word64 res[3])
       str += 8;
     }
 
-#ifdef BIG_ENDIAN
+#ifdef ARCH_BIG_ENDIAN
   for(j=0; j<i; j++)
     temp[j^7] = ((byte*)str)[j];
 
@@ -753,7 +749,7 @@ word64 table[4*256] = {
 
 void swap_digest(unsigned char *digest)
 {
-  #ifdef BIG_ENDIAN
+  #ifdef ARCH_BIG_ENDIAN
   {
     int i,j; 
     for(i=0; i<3; i++){
@@ -779,7 +775,7 @@ char hexa(int i)
 #define MAX_TIGER_CHUNK_SIZE 1024
 static word64 tiger_buffer[MAX_TIGER_CHUNK_SIZE];
 
-void tiger_hash(char prefix, char *s, int len, unsigned char *digest)
+void tiger_hash(char prefix, char *s, OFF_T len, unsigned char *digest)
 {
   char *buffer = (char*) tiger_buffer;
   word64 ndigest[3];
@@ -794,10 +790,10 @@ void tiger_hash(char prefix, char *s, int len, unsigned char *digest)
   swap_digest(digest);
 }
 
-void tiger_tree_string(char *s, int len, int pos, int block_size, char *digest)
+void tiger_tree_string(char *s, size_t len, size_t pos, size_t block_size, char *digest)
 {
   if(block_size == BLOCK_SIZE){
-    int length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
+    size_t length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
     tiger_hash(0, s+pos, length, digest);
   } else {    
     if(pos+block_size/2 >=len){
@@ -820,16 +816,11 @@ void tiger_tree_string(char *s, int len, int pos, int block_size, char *digest)
 
 ***************************************************************************/
 
-#include "tiger.h"
 #include "md4.h"
 
-#include "caml/mlvalues.h"
-#include "caml/fail.h"
-#include "caml/alloc.h"
-
-int tiger_block_size(int len)
+OFF_T tiger_block_size(OFF_T len)
 {
-  int block_size = BLOCK_SIZE;
+  OFF_T block_size = BLOCK_SIZE;
   while(block_size < len) block_size *= 2;
   return block_size;
 }
