@@ -34,7 +34,8 @@ let max_buffered = ref (Int64.of_int (1024 * 1024))
 let create_file_mode = ref 0o664
 let create_dir_mode = ref 0o755
 let verbose = ref false
-let max_cache_size = ref 50
+let max_cache_size_default = 50
+let max_cache_size = ref max_cache_size_default
   
 let mini (x: int) (y: int) =
   if x > y then y else x
@@ -116,7 +117,10 @@ module FDCache = struct
           | Some fd -> fd
           | None ->
 	      if !cache_size >= !max_cache_size then close_one ();
-	      let fd =
+	      let fd = Unix2.tryopen_umask 0 (fun _old_umask ->
+		(* disable umask restrictions temporarily; 
+		   rights on downloaded files should be dictated 
+		   by create_file_mode instead *)
 		try
 		  if t.writable then
                     Unix.openfile t.filename 
@@ -129,7 +133,7 @@ module FDCache = struct
 		    t.filename 
 		    (if t.writable then "rw" else "ro") 
                     (Printexc2.to_string e);
-		  raise e
+		  raise e)
 	      in
 		incr cache_size;
 (*              lprintf "local_force: opening %d\n" (Obj.magic fd_rw);  *)
@@ -1671,7 +1675,7 @@ type statfs = {
   f_files : int64;  (* total file nodes in file system *)
   f_ffree : int64;  (* free file nodes in fs *)
   f_fsid : unit;  (* See note in statfs(2) *)
-  f_fnamelen : int64; (* maximum length of filenames *)
+  f_fnamelen : int64; (* maximum length of filenames, maximum Sys.max_string_length *)
   f_basetype : string; (* type of filesystem - Solaris, (-1) on other systems, use f_type there *)
   f_frsize : int64;  (* Fundamental file system block size, (-1) if not provided by system *)
 }
@@ -1700,26 +1704,22 @@ let bsize dir =
 
 let blocks dir =
   try
-    let s = statfs dir in
-    Some s.f_blocks
+    Some (statfs dir).f_blocks
   with e -> None
 
 let bfree dir =
   try
-    let s = statfs dir in
-    Some s.f_bfree
+    Some (statfs dir).f_bfree
   with e -> None
 
 let bavail dir =
   try
-    let s = statfs dir in
-    Some s.f_bavail
+    Some (statfs dir).f_bavail
   with e -> None
 
 let fnamelen dir =
   try
-    let s = statfs dir in
-    Some s.f_fnamelen
+    Some (Int64.to_int (min (statfs dir).f_fnamelen (Int64.of_int Sys.max_string_length)))
   with e -> None
 
 let disktotal dir =
@@ -1836,6 +1836,12 @@ let filesystem dir =
 	     Printf.sprintf "unknown (%Ld)" s.f_type
   with e -> "not supported"
 
+let set_max_cache_size v =
+  max_cache_size := v;
+  while !FDCache.cache_size > !max_cache_size do FDCache.close_one () done
+
+let get_max_cache_size () = !max_cache_size
+
 let _ =
   Heap.add_memstat "Unix32" (fun level buf ->
       let counter = ref 0 in
